--           ,   ISBN
-- : mr. Aleph <mraleph at gorodok dot net>

local function UseCache()
  return useCache == "true"
end

local http  = require "socket.http"
local ltn12 = require "ltn12"
local mime  = require "mime"

local books = require "books"

local print, pcall = print, pcall

local ipairs, type, assert, select, pairs = ipairs, type, assert, select, pairs

local io    = require "io"
local table = require "table"

local function getPicturesDir()
  return picturesDir or "."
end


module(...)

local Book, BookArray = books.Book, books.BookArray

local urlSearch1 = "http://www.ozon.ru/?context=advsearch_book&ISBN=%s"
local urlSearch2 = "http://www.ozon.ru/?context=search&text=%s"
local urlItem   = "http://www.ozon.ru/context/detail/id/%s/?type=1"
local urlOzon   = "http://www.ozon.ru"

local function maskUrl(url)
  return url:gsub("[^%w]", "_")
end

local function fromCache(url) 
  if not UseCache() then return nil end
  
  local t = {}
  local f = io.open(maskUrl(url), "r")
  if f then 
    for l in f:lines() do
      table.insert(t, l)
    end    
    f:close()    
  else
    return nil
  end
  return t
end

local function toCache(url, t)   
  if not UseCache() then return end
  
  local f = io.open(maskUrl(url), "w")
  for _, s in ipairs(t) do
    f:write(s, '\n')
  end    
  f:close()  
end


local function split(s)
  local t = {}
  s = mime.normalize "\n" (s)
  for l in s:gmatch "([^\n]*)\n" do
    table.insert(t, l)
  end
  table.insert(t, s:match "\n([^\n]*)$")
  return t
end

local requestHeaders = {
  ["User-Agent"] = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.51";
  ["Host"] = "www.ozon.ru";
  ["Accept"] = "text/html";
  ["Accept-Language"] = "en";
  ["Accept-Charset"] = "windows-1252";
  ["Accept-Encoding"] = "identity";
  ["Connection"] = "Keep-Alive"
}

local function request(url)  
  local t = fromCache(url)
  
  if t then    
    return t
  end  
  
  t = {}
  
  local _, s, code, headers, status = pcall(http.request, {  
                                                  url = url
                                                  , redirect = false
                                                  , sink = ltn12.sink.table(t)
                                                  , headers = requestHeaders
                                                })

  if _ == nil or s == nil then
    return nil 
  end

--[[
  print "Dumping headers ---------------------"
  table.foreach(headers, print)
  print "-------------------------------------"
  print (status, s, code)

  if not status then print(s) end
--]]
  
  if code == 301 or code == 302 then
    s = headers.location
  else
    s = split(table.concat(t))
    toCache(url, s)  
  end  
  
  return s
end

local function saveImage(id, url)
  local ext      = url:match "(%.%w*)$"
  local filename = ("%s/%s%s"):format(getPicturesDir(), id, ext) 
  local f        = io.open(filename, "wb")
  http.request {url = url, redirect = false, sink = ltn12.sink.file(f)}
  return filename
end

function table.insert(to, ...)  
  local l = #to  
  for i = 1, select('#', ...) do to[l + i] = select(i, ...) end
end


local function parseAuthorPage(page)
  local t = request(page)
  
  if not t then return nil end
  
  assert(type(t) == "table")
  
  local function match(pattern)
    for i = 1, #t do
      if t[i]:find(pattern) then                
        return t[i]:match(pattern)
      end
    end    
    return nil
  end
  
  local function mmatch(fromPattern, toPattern)
    local matches = {}
    for i = 1, #t do
      if t[i]:find(fromPattern) then        
        matches = { t[i]:match(fromPattern) }        
        for j = i + 1, #t do
          if t[j]:find(toPattern) then
            table.insert(matches, t[j]:match(toPattern))
            break
          end
          table.insert(matches, t[j])
        end
        return table.concat(matches, '\n')
      end
    end    
    return nil
  end
  
  local info = {}
  
  info.name, info.orig, info.years = match '<td colspan="2"><BIG class="big2">(.-)</BIG><BR>(.-)<BR>(.-)<BR><BR></td>'
  
  if not info.name then
    info.name, info.years = match '<td colspan="2"><BIG class="big2">(.-)</BIG><BR>(.-)<BR><BR></td>'
  end
  
  if not info.name then
    info.name, info.orig  = match '<td colspan="2"><BIG class="big2">(.-)</BIG><BR>(.-)<BR><BR><BR></td>'
  end
  
  if not info.name then return nil end
  
  info.imageUrl = match "swapImages%(document%.cover%,'.-'%,'(.-)'%,%d*%,%d*%)"
  
  if info.imageUrl then    
    info.imageFile = saveImage('a'..(page:match '/(%d*)/$'), urlOzon .. info.imageUrl)    
  end
  
  info.bio = match '<a onclick="return swapImages.-</a>(.-)<BR clear="all"></DIV>'
  
  if not info.bio then
    info.bio = mmatch ('<a onclick="return swapImages.-</a>(.-)'
                       , '(.*)<BR clear="all"></DIV>')
  end
   
  return info
end

local function parseBookPage(id)    
  local t = request(urlItem:format(id))
  
  if not t then return nil end
  
  assert(type(t) == "table")
  
  local function match(pattern)
    for i = 1, #t do
      if t[i]:find(pattern) then                
        return t[i]:match(pattern)
      end
    end    
    return nil
  end
  
  local function mmatch(fromPattern, toPattern)
    local matches = {}
    for i = 1, #t do
      if t[i]:find(fromPattern) then        
        matches = { t[i]:match(fromPattern) }        
        for j = i + 1, #t do
          if t[j]:find(toPattern) then
            table.insert(matches, t[j]:match(toPattern))
            break
          end
          table.insert(matches, t[j])
        end
        return table.concat(matches, '\n')
      end
    end    
    return nil
  end
  
  local origtitle
  
  local authorsStr, title, series = match '<a href="#persons" class="bigger">(.-)<br></a><big class="big2">(.-)</big><br><[df].-color.-#666666;?">: <a href="[^"]*" title="" class="norm">(.-)</a>'

  if not authorsStr then       
    authorsStr, title, origtitle, series = match '<a href="#persons" class="bigger">(.-)<br></a><big class="big2">(.-)</big><br><[df].-color.-#666666;?">(.-)<br>: <a href=".-" title="" class="norm">(.-)</a>'    
  end  

  if not authorsStr then       
    authorsStr, title, series = match '<a href="#persons" class="bigger">(.-)<br></a><big class="big2">(.-)</big><br><[df].-color.-#666666;?">: <a href=".-" title="" class="norm">(.-)</a>'    
  end    
  
  if not authorsStr then 
    authorsStr, title, origtitle = match '<a href="#persons" class="bigger">(.-)<br></a><big class="big2">(.-)</big><br><[df].-color.-#666666;?">(.-)<br>'    
  end
  
  if not authorsStr then
    authorsStr, title = match '<a href="#persons" class="bigger">(.-)<br></a><big class="big2">(.-)</big>'
  end

  if not title then       
    title, origtitle, series = match '<big class="big2">(.-)</big><br><[df].-color.-#666666;?">(.-)<br>: <a href=".-" title="" class="norm">(.-)</a>'    
  end  

  if not title then       
    title, series = match '<big class="big2">(.-)</big><br><[df].-color.-#666666;?">: <a href=".-" title="" class="norm">(.-)</a>'    
  end  
  
  if not title then 
    title, origtitle = match '<big class="big2">(.-)</big><br><[df].-color.-#666666;?">(.-)<br>'    
  end
  
  if not title then
    title = match '<big class="big2">(.-)</big><br>'
  end  
  
  local imageUrl = match "swapImages%(document%.cover%,'.-'%,'(.-)'%,%d*%,%d*%)"
  
  if imageUrl then    
    imageFile = saveImage(id, urlOzon..imageUrl)
  end
  
  local description = match '<tr><td colspan="2" class="detail_leftcol"><b> </b></td></tr><tr><td>&nbsp;</td><td class="vertpadd">(.-)</td></tr>$'
  
  if not description then    
    description = mmatch ('<tr><td colspan="2" class="detail_leftcol"><b> </b></td></tr><tr><td>&nbsp;</td><td class="vertpadd">(.-)$'
                          , '(.*)</td></tr>')        
  end
  
  local publishers, type                                                        
  local publisher, year, bookCover, pages, circulation, format  = match '<[fd].-color.-#666666;?">: <[Aa] href=".-" title="" class="norm">(.-)</[Aa]>, (.-)<br>(.-), (.-) %.<br>.-<br><font style="white%-space: nowrap;">: (.-) .</font><br><a href=".-" title="  "></a>: ([^<]*)'
  
  if not publisher then
    type, publisher, year, bookCover, pages, circulation, format  = match '<[fd].-color.-#666666;?">(.-)<br><br>: <[Aa] href=".-" title="" class="norm">(.-)</[Aa]>, (.-)<br>(.-), (.-) %.<br>.-<br><font style="white%-space: nowrap;">: (.-) .</font><br><a href=".-" title="  "></a>: ([^<]*)'
  end
  
  if not publisher then
    publishers, year, bookCover, pages, circulation, format  = match '<[fd].-color.-#666666;?">: (.-), ([^,]-)<br>(.-), (.-) %.<br>.-<br><font style="white%-space: nowrap;">: (.-) .</font><br><a href=".-" title="  "></a>: ([^<]*)'
    if publishers then
      publishers = publishers:gsub("<.->", "")
    end
  end
  
  local captures = { 
           AuthorsStr      = authorsStr
           , Title         = title
           , Series        = series
           , OriginalTitle = origtitle
           , imageUrl      = imageUrl
           , imageFile     = imageFile
           , Description   = description
           , publisher     = publisher
           , publishers    = publishers
           , year          = year
           , bookCover     = bookCover
           , pages         = pages
           , circulation   = circulation
           , format        = format
           , type          = type
         }
  
  local authors = match '<a name="persons"></a><b>?</b>&nbsp;</td></tr>(.*)$'
  
  if not authors then
    authors = mmatch('<a name="persons"></a>(.*)$', '^%s*$')      
  end
  
  captures.authors = 0  

  local function addAuthor(page)
    local author = parseAuthorPage("http://www.ozon.ru"..page)
    
    if author then
      captures.authors = captures.authors + 1      
      for k, v in pairs(author) do
        captures['Author'..captures.authors..'_'..k] = v
      end      
    end
  end
  
  if authors then 
    local fallback = true
    for page, role in authors:gmatch '<a href="([^"]-)" class="bigger">.-</a></big>%s*%((.-)%)' do
      if role:find "" then
        addAuthor (page)
      else
        fallback = false
      end
    end 
   
    if (captures.authors == 0) and fallback then
      -- nobody marked found?
      -- fallback to the simple matching
      for page in authors:gmatch '<a href="([^"]-)" class="bigger">.-</a>' do
        addAuthor (page)
      end
    end
  end
  
  return Book:new(captures) 
end

local function trySearch(url)
  local t = request(url)

  if not t then return nil end
  
  if type(t) == "string" then    
    local id = t:match '/(%d*)/$'    
    return parseBookPage(id)
  end  
  
  local candidates = BookArray:new(ISBN)
  for _, l in ipairs(t) do
    local id = l:match '<a name="buy(.*)"></a>'
    if id then
      table.insert(candidates, parseBookPage(id))
    end
  end

  if #candidates > 0 then
    return candidates  
  end
  
  return nil
end

function getInfo(ISBN)    
  return trySearch(urlSearch1:format (ISBN)) or trySearch(urlSearch2:format (ISBN)) 
end

